@@ -1,20 +1,10 @@
-package Devel::Command::DBSub::DB_5_8_5;
+package Devel::Command::DBSub::DB_5_8_6;
 
 sub import {
-  if (my($sub_version) = ($] =~ /^5.008(.*)/)) {
-    # This module might work.
-    if ($sub_version+0 <= 5) {
-      return \&DB::alt_56_DB;
-    }
-    else {
-      # Not 5.8.5 or less.
-      return undef;
-    }
-  }
-  else {
-    # not 5.8 at all.
-    return undef;
-  }
+  # $] must be greater than 5.8.5, but still a 5.8 Perl
+  $] > 5.008005 and $] =~ /^5\.008/
+    ? \&DB::alt_586_DB
+    : undef;
 }
 
 {
@@ -22,7 +12,17 @@ no strict;
 no warnings;
 package DB;
 
-sub DB {
+sub alt_586_DB {
+
+    # lock the debugger and get the thread id for the prompt
+	lock($DBGR);
+	my $tid;
+	if ($ENV{PERL5DB_THREADED}) {
+		$tid = eval { "[".threads->self->tid."]" };
+	}
+	else {
+	        $tid = "";
+	}
 
     # Check for whether we should be running continuously or not.
     # _After_ the perl program is compiled, $single is set to 1:
@@ -182,7 +182,6 @@ EOP
               . "package $package;";    # this won't let them modify, alas
         } ## end elsif ($package eq 'DB::fake')
 
-
         else {
 
             # Still somewhere in the midst of execution. Set up the
@@ -246,7 +245,6 @@ EOP
         } ## end else [ if ($slave_editor)
     } ## end if ($single || ($trace...
 
-
     # If there's an action, do it now.
     $evalarg = $action, &eval if $action;
 
@@ -291,7 +289,7 @@ EOP
             # ... and we got a line of command input ...
             defined(
                 $cmd = &readline(
-                        "$pidprompt  DB"
+                        "$pidprompt $tid DB"
                       . ( '<' x $level )
                       . ( $#hist + 1 )
                       . ( '>' x $level ) . " "
@@ -300,6 +298,7 @@ EOP
           )
         {
 
+			defined \&share and share($cmd);
             # ... try to execute the input as debugger commands.
 
             # Don't stop running.
@@ -319,6 +318,8 @@ EOP
             chomp($cmd);    # get rid of the annoying extra newline
             push( @hist, $cmd ) if length($cmd) > 1;
             push( @truehist, $cmd );
+			share(@hist);
+			share(@truehist);
 
             # This is a restart point for commands that didn't arrive
             # via direct user input. It allows us to 'redo PIPE' to
@@ -356,7 +357,7 @@ EOP
                foreach my $do (keys %DB::commands) {
                  next unless $cmd =~ /^$do\s*/;
                  $commands{$do}->($cmd) and next CMD;
-                 #  ? next CMD : last CMD;
+		 #  ? next CMD : last CMD;
                }
 
                 $cmd =~ /^q$/ && do {
@@ -395,10 +396,8 @@ EOP
                     next CMD;
                 };
 
-
                 $cmd =~ s/^X\b/V $package/;
 
-
                 # Bare V commands get the currently-being-debugged package
                 # added.
                 $cmd =~ /^V$/ && do {
@@ -549,12 +548,11 @@ EOP
 
                 # All of these commands were remapped in perl 5.8.0;
                 # we send them off to the secondary dispatcher (see below).
-                $cmd =~ /^([aAbBhilLMoOPvwW]\b|[<>\{]{1,2})\s*(.*)/so && do {
+                $cmd =~ /^([aAbBeEhilLMoOPvwW]\b|[<>\{]{1,2})\s*(.*)/so && do {
                     &cmd_wrapper( $1, $2, $line );
                     next CMD;
                 };
 
-
                 $cmd =~ /^y(?:\s+(\d*)\s*(.*))?$/ && do {
 
                     # See if we've got the necessary support.
@@ -620,7 +618,6 @@ EOP
                     last CMD;
                 };
 
-
                 # c - start continuous execution.
                 $cmd =~ /^c\b\s*([\w:]*)\s*$/ && do {
 
@@ -1107,6 +1104,24 @@ EOP
                 $cmd =~ /^(R|rerun\s*(.*))$/ && do {
                     my @args = ($1 eq 'R' ? restart() : rerun($2));
 
+                    # Close all non-system fds for a clean restart.  A more
+                    # correct method would be to close all fds that were not
+                    # open when the process started, but this seems to be
+                    # hard.  See "debugger 'R'estart and open database
+                    # connections" on p5p.
+
+                    my $max_fd = 1024; # default if POSIX can't be loaded
+                    if (eval { require POSIX }) {
+                        $max_fd = POSIX::sysconf(POSIX::_SC_OPEN_MAX());
+                    }
+
+                    if (defined $max_fd) {
+                        foreach ($^F+1 .. $max_fd-1) {
+                            next unless open FD_TO_CLOSE, "<&=$_";
+                            close(FD_TO_CLOSE);
+                        }
+                    }
+
                     # And run Perl again.  We use exec() to keep the
                     # PID stable (and that way $ini_pids is still valid).
                     exec(@args) || print $OUT "exec failed: $!\n";
@@ -1205,7 +1220,6 @@ EOP
             }
         } ## end while (($term || &setterm...
 
-
         continue {    # CMD:
 
             # At the end of every command:
@@ -1286,7 +1300,7 @@ __END__
 
 =head1 NAME
 
-Devel::Command::DBSub::DB_5_8_5 - Devel::Command debugger patch for 5.8.5 and up
+Devel::Command::DBSub::DB_5_8_6 - Devel::Command debugger patch for 5.8.6 and up
 
 =head1 SYNOPSIS
 
@@ -1295,8 +1309,14 @@ Devel::Command::DBSub::DB_5_8_5 - Devel::Command debugger patch for 5.8.5 and up
 
 =head1 DESCRIPTION
 
-C<Devel::Command::DBSub::DB_5_8_5> loads a patched version of the debugger's
-C<DB()> routine that will work with Perl 5.8.0 through Perl 5.8.5.
+C<Devel::Command::DBSub::DB_5_8_6> loads a patched version of the debugger's
+C<DB()> routine that will work with Perl 5.8.6 and up.
+
+=head1 NOTE
+
+The POD documentation was removed from this subroutine to prevent this 
+documentation becoming confusing. If you want real docs on how the debugger
+works, see C<perldoc perl5db.pl>.
 
 =head1 SEE ALSO
 
